from sbgsdk.schema.io_classes import IOAttr, SchemaBased, IODef, SchemaBased
from sbgsdk.schema.io_params import BoolAttr, EnumAttr, IntAttr, RealAttr, StringAttr, StructAttr

Input = Output = IOAttr
Params = SchemaBased
Inputs = Outputs = IODef